﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.SilverlightEdition;
using ThinkGeo.MapSuite.SilverlightCore;
using System.Windows.Shapes;

namespace CSharp_Samples
{
    public partial class HoverPopupOfMarker : UserControl
    {
        public HoverPopupOfMarker()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(LabelRotateAMarker_Loaded);
        }

        private void LabelRotateAMarker_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            WorldMapKitWmsSilverlightOverlay baseOverlay = new WorldMapKitWmsSilverlightOverlay();
            Map1.Overlays.Add(baseOverlay);

            SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
            Marker marker = new Marker(-94.558, 39.078);
            marker.ImageSource = new BitmapImage(new Uri("/theme/marker_red_shadow.png", UriKind.RelativeOrAbsolute));
            marker.ImageOffsetX = -10;
            marker.ImageOffsetY = -25;

            ControlTemplate template = (ControlTemplate)LayoutRoot.Resources["PopupTemplate"];
            marker.Popup = new CusGeoPopup();
            marker.Popup.Visibility = Visibility.Collapsed;
            marker.PopupDelay = new TimeSpan(2000);
            marker.Popup.Template = template;

            GeneratePopupContent(marker.Popup);
            markerOverlay.Markers.Add(marker);

            Map1.Overlays.Add("SimpleMarkerOverlay", markerOverlay);
            Map1.CurrentExtent = new RectangleShape(-131.22, 55.05, -54.03, 16.91);
        }

        private void GeneratePopupContent(GeoPopup popup)
        {
            popup.Width = 610;
            popup.Height = 230;
            popup.Offset = new Point(-400, -30);

            Image img = new Image();
            img.Source = new BitmapImage(new Uri("http://upload.wikimedia.org/wikipedia/commons/thumb/a/a9/Douglas_county_kansas_courthouse.jpg/250px-Douglas_county_kansas_courthouse.jpg"));
            img.Width = 250;
            img.Height = 188;

            TextBlock text = new TextBlock();
            text.SetValue(Canvas.LeftProperty, 255d);
            text.Width = 330;
            text.TextWrapping = TextWrapping.Wrap;
            text.FontSize = 10d;
            StringBuilder content = new StringBuilder();
            content.Append("  Lawrence is a city in Northeastern Kansas")
                .Append("in the United States. Lawrence serves as the county seat of Douglas County, Kansas. Located 41 miles west ")
                .Append("of Kansas City, Lawrence is situated along the banks of the Kansas (Kaw) and Wakarusa Rivers. It is considered ")
                .Append("governmentally independent and is the principal city within the Lawrence, Kansas, Metropolitan Statistical Area, ")
                .Append("which encompasses all of Douglas County. As of the 2000 census, the city had a population of 80,098, making it ")
                .Append("the sixth largest city in Kansas. 2006 estimates[3] place the city's population at 89,110. A quintessential")
                .Append("college town, Lawrence is home to The University of Kansas and Haskell Indian Nations University.");
            text.Text = content.ToString();

            HyperlinkButton moreInfo = new HyperlinkButton();
            moreInfo.SetValue(Canvas.TopProperty, 190d);
            moreInfo.Content = "more about Lawrence city...";
            moreInfo.Foreground = new SolidColorBrush(Colors.Blue);
            moreInfo.NavigateUri = new Uri("http://en.wikipedia.org/wiki/Lawrence%2C_Kansas");
            Canvas container = new Canvas();
            container.Margin = new Thickness(5);
            container.Children.Add(img);
            container.Children.Add(text);
            container.Children.Add(moreInfo);

            popup.Content = container;
        }
    }

    public class CusGeoPopup : GeoPopup
    {
        private Canvas popupContainer;
        private Border popupBorder;
        private Polyline popupArrow;

        public CusGeoPopup()
            : base()
        { }

        public override void OnApplyTemplate()
        {
            popupContainer = (Canvas)GetTemplateChild("popupContainer");
            popupArrow = (Polyline)GetTemplateChild("popupArrow");
            popupBorder = (Border)GetTemplateChild("popupBorder");

            Redraw();
        }

        public new void Redraw()
        {
            if (popupContainer != null && popupBorder != null)
            {
                if (Offset.Y < 0)
                {
                    double offsetX = Offset.X;
                    double offsetY = Height * -1 + Offset.Y;
                    popupContainer.Margin = new Thickness(offsetX, offsetY, 0, 0);
                    popupArrow.Points[0] = new Point(Width * 1 / 3, Height - BorderThickness.Left);
                    popupArrow.Points[1] = new Point(offsetX * -1, offsetY * -1);
                    popupArrow.Points[2] = new Point(Width * 1 / 2, Height - BorderThickness.Left);
                }
                else if (Offset.Y > 0)
                {
                    double offsetX = Offset.X;
                    double offsetY = Offset.Y;
                    popupContainer.Margin = new Thickness(0, 0, 0, 0);
                    popupArrow.Points[0] = new Point(Width * 1 / 3, offsetY + BorderThickness.Left);
                    popupArrow.Points[1] = new Point(offsetX * -1, 0);
                    popupArrow.Points[2] = new Point(Width * 1 / 2, offsetY + BorderThickness.Left);
                    popupBorder.SetValue(Canvas.LeftProperty, offsetX);
                    popupBorder.SetValue(Canvas.TopProperty, offsetY);
                }
                popupArrow.StrokeThickness = BorderThickness.Left;
            }
        }
    }
}
